Macro rocket_include_tera::tera_response [−][src]
Used for retrieving and rendering the file you input through the macro tera_resources_initialize! as a TeraResponse instance with rendered HTML. When its respond_to method is called, three HTTP headers, Content-Type, Content-Length and Etag, will be automatically added, and the rendered HTML can optionally not be minified.