Macro rocket_include_tera::tera_response_cache [−][src]
Used for wrapping a TeraResponse and its constructor, and use a key to cache its HTML and ETag in memory. The cache is generated only when you are using the release profile.